
-- Globals
Fonts={}
Pics={}
Orca={}
Floes={}
LeftFloe=1
RightFloe=1
SeaFloor=3000
Seals={}
LeftSeal=1
RightSeal=1
Score=0
Bubbles={}
BloodBlots={}
IceShards={}
SeaWeeds={}
tau=2*math.pi

-- Locals
local FullScreen=false-- Change to true to start directly in fullscreen, without having to press F
local ScreenShotting=false-- Make the game go slower, make right click aim at a given height, and fix window size to 548x548

local function CoinFlip()
	return math.random()>0.5
end

local function sign(x)
	if x>0 then
		return 1
	elseif x<0 then
		return -1
	else
		return 0
	end
end

local function InvertAngle(r)
	local r=(r+tau/4)%tau
	if r>tau/2 then
		return (7*tau/4-r)%tau
	else
		return (7*tau/4-r)%tau
	end
end

local function NewAnimal(kind,x,y,size)
	local pic=Pics[kind]
	assert(pic,"Cannot find graphic for \""..kind.."\"")
	local sx,sy=pic:getDimensions()
	local scale=math.min(size/sx,size/sy)
	local facing=CoinFlip() and 1 or -1
	return {kind=kind,pic=pic,x=x,y=y,r=facing>0 and 0 or math.pi,size=size,scale=scale,ox=sx/2,oy=sy/2,speed=0,dx=0,dy=50,facing=facing}
end

local function CreateFloes()
	local f={}
	local size=math.random(5,500)
	table.insert(f,{-size/2,size/2})
	local xmin,xmax=-size/2,size/2
	while xmax<1234567 do
		local size=math.random(5,500)
		local dist=math.random(2,30)^2
		table.insert(f,{xmax+dist,xmax+dist+size})
		xmax=xmax+dist+size
	end
	while xmin>-1234567 do
		local size=math.random(5,500)
		local dist=math.random(2,30)^2
		table.insert(f,1,{xmin-dist-size,xmin-dist})
		xmin=xmin-dist-size
	end
	for _,f in ipairs(f) do
		local size=f[2]-f[1]
		local x=(f[1]+f[2])/2
		thickness=7+14*math.random()
		tilt=math.atan((1.5*thickness*math.random()-thickness/2)/size)
		f.x1=x-size/2*math.cos(tilt)+thickness/2*math.sin(tilt)
		f.y1=0-size/2*math.sin(tilt)-thickness/2*math.cos(tilt)
		f.x2=x-size/2*math.cos(tilt)-thickness/2*math.sin(tilt)
		f.y2=0-size/2*math.sin(tilt)+thickness/2*math.cos(tilt)
		f.x3=x+size/2*math.cos(tilt)-thickness/2*math.sin(tilt)
		f.y3=0+size/2*math.sin(tilt)+thickness/2*math.cos(tilt)
		f.x4=x+size/2*math.cos(tilt)+thickness/2*math.sin(tilt)
		f.y4=0+size/2*math.sin(tilt)-thickness/2*math.cos(tilt)
	end
	return f
end

local function AdjustFloesBoundary()
	local x=Orca.x
	local wx=love.window.getWidth()/2+20
	while LeftFloe>=2 and Floes[LeftFloe-1][2]>x-wx do
		LeftFloe=LeftFloe-1
	end
	while Floes[LeftFloe][2]<x-wx and LeftFloe~=#Floes do
		LeftFloe=LeftFloe+1
	end
	while RightFloe<#Floes and Floes[RightFloe+1][1]<x+wx do
		RightFloe=RightFloe+1
	end
	while Floes[RightFloe][1]>x+wx and RightFloe~=1 do
		RightFloe=RightFloe-1
	end
end

local function FindBoundaryFloes()
	local x=Orca.x
	local f=math.ceil(#Floes/2)
	local i=2
	while i<#Floes do
		i=i*2
	end
	repeat
		if Floes[f][2]<x then
			f=math.min(#Floes,f+i)
		elseif Floes[f][1]>x then
			f=math.max(1,f-i)
		end
		i=math.floor(i/2)
	until i==0
	LeftFloe=f
	RightFloe=f
	AdjustFloesBoundary()
end


local function FindFloeAt(x)
	local f=math.ceil(#Floes/2)
	local i=2
	while i<#Floes do
		i=i*2
	end
	repeat
		if Floes[f][2]<x then
			f=math.min(#Floes,f+i)
		elseif Floes[f][1]>x then
			f=math.max(1,f-i)
		end
		i=math.floor(i/2)
	until i==0
	local str=""
	while Floes[f][1]>x and f>=2 do
		f=f-1
		str=str.."inc"
	end
	while Floes[f][2]<x and f~=#Floes do
		f=f+1
		str=str.."dec"
	end
	if x>=Floes[f][1] and x<=Floes[f][2] then
		return Floes[f],f,str
	else
		return nil,nil,str
	end
end

local function ActivateSeal(a)
	a.active=100
	a.speed=0
	a.dx=0
	a.dy=0
end

local function DeactivateSeal(a)
	a.active=nil
	a.floe=nil
	a.speed=nil
	a.dx=nil
	a.dy=nil
end

local function BreakFloe(n,x,holesize)
	for s=LeftSeal,RightSeal do
		local sf=Seals[s].floe
		if not sf then
			_,sf=FindFloeAt(Seals[s].x)
		end
		if sf and sf==n then
			ActivateSeal(Seals[s])
		end
	end
	local nf={{Floes[n][1],math.min(Floes[n][2],x-holesize/2)},{math.max(Floes[n][1],x+holesize/2),Floes[n][2]}}
	thickness=math.sqrt((Floes[n].x2-Floes[n].x1)^2+(Floes[n].y2-Floes[n].y1)^2)
	for k,f in ipairs(nf) do
		local size=f[2]-f[1]
		if size>1 then
			local x=(f[1]+f[2])/2
			tilt=math.atan((1.5*thickness*(k-1)-thickness/2)/size)
			f.x1=x-size/2*math.cos(tilt)+thickness/2*math.sin(tilt)
			f.y1=0-size/2*math.sin(tilt)-thickness/2*math.cos(tilt)
			f.x2=x-size/2*math.cos(tilt)-thickness/2*math.sin(tilt)
			f.y2=0-size/2*math.sin(tilt)+thickness/2*math.cos(tilt)
			f.x3=x+size/2*math.cos(tilt)-thickness/2*math.sin(tilt)
			f.y3=0+size/2*math.sin(tilt)+thickness/2*math.cos(tilt)
			f.x4=x+size/2*math.cos(tilt)+thickness/2*math.sin(tilt)
			f.y4=0+size/2*math.sin(tilt)-thickness/2*math.cos(tilt)
		end
	end
	table.remove(Floes,n)
	for k=#nf,1,-1 do
		if nf[k][2]-nf[k][1]>12 then
			table.insert(Floes,n,nf[k])
		end
	end
	IceShards:setPosition(x,0)
	IceShards:emit(math.random(2,7))
end


local function SpawnSeals()
	for _,f in ipairs(Floes) do
		if math.random()<0.4 then
			local xmin,xmax=f[1],f[2]
			local size=xmax-xmin
			local n=math.ceil(1*size/100)
			for k=1,n do
				if math.random()<0.3 then
					local x=size/(n+1)*k-20+40*math.random()
					local y=(f.y3-f.y2)*x/size+f.y1-5
					table.insert(Seals,NewAnimal("seal",xmin+x,y,50))
					Seals[#Seals].r=Seals[#Seals].r+(2*math.random()-1)*tau/16
				end
			end
		end
	end
end

local function AdjustSealsBoundary()
	local OldLeft,OldRight=LeftSeal,RightSeal
	local x=Orca.x
	local wx=4000
	for s=LeftSeal,RightSeal do
		if Seals[s].active and math.abs(Seals[s].x-x)>wx then
			table.remove(Seals,s)
		end
	end
	while LeftSeal>=2 and Seals[LeftSeal-1].x>x-wx do
		LeftSeal=LeftSeal-1
	end
	while Seals[LeftSeal].x<x-wx and LeftSeal~=#Seals do
		LeftSeal=LeftSeal+1
	end
	while RightSeal<#Seals and Seals[RightSeal+1].x<x+wx do
		RightSeal=RightSeal+1
	end
	while Seals[RightSeal].x>x+wx and RightSeal~=1 do
		RightSeal=RightSeal-1
	end
end

local function FindBoundarySeals()
	local x=Orca.x
	local a=math.ceil(#Seals/2)
	local i=2
	while i<#Seals do
		i=i*2
	end
	repeat
		a=Seals[a].x<x and math.min(#Seals,a+i) or math.max(1,a-i)
		i=math.floor(i/2)
	until i==0
	LeftSeal=a
	RightSeal=a
	AdjustSealsBoundary()
end


local function GetOrcaMouth()
	local x=Orca.x+80*math.cos(Orca.r)-Orca.facing*15*math.sin(Orca.r)
	local y=Orca.y+80*math.sin(Orca.r)+Orca.facing*15*math.cos(Orca.r)
	return x,y
end

local function UpdateScreenMode()
	if FullScreen then
		-- Fullscreen
		local AvailableFullScreenModes=love.window.getFullscreenModes()
		table.sort(AvailableFullScreenModes,function(a,b) return a.width*a.height>b.width*b.height end)
		local Biggest=AvailableFullScreenModes[1]
		love.window.setMode(Biggest.width,Biggest.height,{fullscreen=true,fullscreentype=(false and "desktop" or "normal"),})
	else
		-- Windowed
		local dsx,dsy=love.window.getDesktopDimensions()
		if ScreenShotting then
			love.window.setMode(548,548,{resizable=false})
		else
			love.window.setMode(dsx*4/5,dsy*4/5,{resizable=true, minwidth=200, minheight=200})
		end
	end
end

local function InitBubbling()
	Bubbles=love.graphics.newParticleSystem(Pics.bubble,100)-- Second param: Max particle count
	Bubbles:setColors(255,255,255,255, 255,255,255,0)-- Up to eight r,g,b,a that particle will go through
	Bubbles:setPosition(Orca.x,Orca.y)-- Position of emitter
	Bubbles:setAreaSpread("normal",20,20)-- The area particles are created in
	Bubbles:setDirection(0)-- Doesn't matter with a 360° spread
	Bubbles:setSpread(tau)-- Spread their direction all around
	Bubbles:setEmissionRate(0)-- How many particles per second
	Bubbles:setEmitterLifetime(-1)-- Emit forever
	Bubbles:setParticleLifetime(1,3)-- min and max in seconds
	Bubbles:setSpeed(2,5)-- Min and max speed
	Bubbles:setLinearAcceleration(0,-8,0,-8)-- xmin,ymin,xmax,ymax 
	Bubbles:setInsertMode("random")
	Bubbles:setRotation(0,0)-- Min and max image rotation of particle in radians
	Bubbles:start()
end

local function InitBleeding()
	BloodBlots=love.graphics.newParticleSystem(Pics.droplet,54321)-- Second param: Max particle count
	BloodBlots:setColors(255,0,0,255, 91,29,29,255, 91,29,29,0)-- Up to eight r,g,b,a that particle will go through
	BloodBlots:setSizes(1,0.2)-- A series of up to eight size
	BloodBlots:setSizeVariation(1)
	BloodBlots:setPosition(0,0)-- Position of emitter
	BloodBlots:setAreaSpread("normal",7,7)-- The area particles are created in
	BloodBlots:setDirection(0)-- Doesn't matter with a 360° spread
	BloodBlots:setSpread(tau)-- Spread their direction all around
	BloodBlots:setEmissionRate(0)-- How many particles per second
	BloodBlots:setEmitterLifetime(-1)-- Emit forever
	BloodBlots:setParticleLifetime(2,4)-- min and max in seconds
	BloodBlots:setSpeed(5,50)-- Min and max speed
	BloodBlots:setLinearAcceleration(0,40,0,40)-- xmin,ymin,xmax,ymax 
	BloodBlots:setInsertMode("random")
	BloodBlots:setRotation(0,tau)-- Min and max image rotation of particle in radians
	BloodBlots:start()
end

local function InitIcesplosion()
	IceShards=love.graphics.newParticleSystem(Pics.iceshard,321)
	IceShards:setColors(255,255,255,255, 255,255,255,255, 255,255,255,0)-- Up to eight r,g,b,a that particle will go through
	IceShards:setSizes(1.2,0.4)-- A series of up to eight size
	IceShards:setSizeVariation(1)
	IceShards:setPosition(0,0)-- Position of emitter
	IceShards:setAreaSpread("normal",7,7)-- The area particles are created in
	IceShards:setDirection(-tau/4)-- Upward
	IceShards:setSpread(tau/4)-- Spread to a quarter of circle
	IceShards:setEmissionRate(0)-- How many particles per second
	IceShards:setEmitterLifetime(-1)-- Emit forever
	IceShards:setParticleLifetime(8,8)-- min and max in seconds
	IceShards:setSpeed(100,300)-- Min and max speed
	IceShards:setLinearAcceleration(0,80,0,160)-- xmin,ymin,xmax,ymax 
	IceShards:setInsertMode("random")
	IceShards:setRotation(0,tau)-- Min and max image rotation of particle in radians
	IceShards:setSpin(-2*tau,2*tau)
	IceShards:start()
end

local function LoadImages()
	love.graphics.setDefaultFilter("linear","linear",2)
	local LoadImg=love.graphics.newImage
	Pics={}
	Pics.orca=LoadImg("orca.png")
	Pics.seal=LoadImg("seal.png")
	Pics.bubble=LoadImg("bubble.png")
	Pics.droplet=LoadImg("droplet.png")
	Pics.iceshard=LoadImg("iceshard.png")
	Pics.seaweed={}
	for k=1,5 do
		Pics.seaweed[k]=LoadImg("seaweed_f"..k..".png")
	end
	Pics.seaweed.quad=love.graphics.newQuad(0,0,23,56,23,56)
end

function love.load(arguments)
	love.window.setTitle("Killer Without Color")
	UpdateScreenMode()
	Fonts[12] = love.graphics.newFont(12)
	Fonts[24] = love.graphics.newFont(24)
	love.mouse.setVisible(true)
	--love.mouse.setGrab(false)
	love.keyboard.setKeyRepeat(true)
	LoadImages()
	Animals={}
	Orca=NewAnimal("orca",0,0,200)
	InitBubbling()
	Floes=CreateFloes()
	FindBoundaryFloes()
	InitIcesplosion()
	SpawnSeals()
	FindBoundarySeals()
	InitBleeding()
end


function love.draw()

	--love.graphics.setBlendMode("additive")
	--love.graphics.setBlendMode("alpha")
	love.graphics.setBlendMode("alpha")

	local p=Orca

	love.graphics.origin()
	local wx,wy=love.window.getWidth(),love.window.getHeight()
	love.graphics.translate(wx/2-p.x,wy/2-p.y)

	-- Never seen
	love.graphics.setBackgroundColor(128,0,128,255)

	-- Sky
	love.graphics.setColor(193,255,255,255)
	love.graphics.polygon("fill",p.x-wx,2,p.x+wx,2,p.x+wx,-2*wy,p.x-wx,-2*wy)

	-- Ice shards
	love.graphics.draw(IceShards)

	-- Water
	for darkness=1,256,5 do
		local y1,y2=(darkness-5)^2/20,darkness^2/20
		love.graphics.setColor(0,128-darkness/2,256-darkness,255)
		love.graphics.polygon("fill",p.x-wx,y1,p.x+wx,y1,p.x+wx,y2,p.x-wx,y2)
	end

	-- Bubbles
	love.graphics.draw(Bubbles)

	-- Sea Floor
	love.graphics.setColor(32,32,0,255)
	love.graphics.polygon("fill",p.x-wx,SeaFloor,p.x+wx,SeaFloor,p.x+wx,SeaFloor+2*wy,p.x-wx,SeaFloor+2*wy)

	-- Sea weeds
	love.graphics.setColor(255,255,255,255)
	for _,sw in ipairs(SeaWeeds) do
		love.graphics.draw(sw.SpriteBatch,0,SeaFloor)
	end

	-- Floes
	for f=LeftFloe,RightFloe do
		local f=Floes[f]
		love.graphics.setColor(230,246,246,255)
		love.graphics.polygon("fill",f.x1,f.y1,f.x2,f.y2,f.x3,f.y3,f.x4,f.y4)
		love.graphics.setColor(0,0,0,255)
		love.graphics.polygon("line",f.x1,f.y1,f.x2,f.y2,f.x3,f.y3,f.x4,f.y4)
	end

	-- Blood
	love.graphics.draw(BloodBlots)

	-- Orca
	love.graphics.setColor(255,255,255,255)
	love.graphics.draw(p.pic,p.x,p.y,p.r,p.scale,p.facing*p.scale,p.ox,p.oy)

	-- Seals
	for a=LeftSeal,RightSeal do
		local a=Seals[a]
		love.graphics.draw(a.pic,a.x,a.y,a.r,a.scale,a.facing*a.scale,a.ox,a.oy)
	end

	if ShowMouth then
		repeat
			local x,y=GetOrcaMouth()
			love.graphics.setColor(255,64,64,255)
			love.graphics.line(x-5,y-5,x+5,y+5)
			love.graphics.line(x-5,y+5,x+5,y-5)
		until true
	end

	-- Instructions
	love.graphics.origin()
	love.graphics.setFont(Fonts[12])
	love.graphics.setColor(255,150,150,255)
	love.graphics.print("Steer with mouse, swim with left mouse button. Press F to toggle fullscreen.",4,wy-14)
	--love.graphics.print("Blood particle count: "..BloodBlots:getCount(),4,wy-30)
	--local _,f,str=FindFloeAt(Orca.x)
	--love.graphics.print("Orca position: x="..math.floor(0.5+Orca.x)..", y="..math.floor(0.5+Orca.y).." Floe:"..tostring(f).." "..str,4,wy-44)
	love.graphics.setFont(Fonts[24])
	love.graphics.print("Score: "..Score,wx-89-Fonts[24]:getWidth(Score),5)

end


function love.keypressed(key)
	if key=="f" then
		FullScreen=not FullScreen
		UpdateScreenMode()
	end
	if key=="escape" then
		if love.event.quit then
			love.event.quit() -- 0.8.0
		else
			love.event.push('q') -- 0.7.2
		end
	end
end


local function UpdateSeaWeeds(dt)
	local CurrentWeed=({3,4,5,4,3,2,1,2})[math.max(1,math.min(8,math.ceil((love.timer.getTime()*3)%8)))]
	local BatchWidth=500
	local wx=love.window.getWidth()
	local xMin=BatchWidth*math.floor((Orca.x-wx)/BatchWidth)
	local xMax=BatchWidth*math.ceil((Orca.x+wx)/BatchWidth)
	for k,sw in ipairs(SeaWeeds) do
		if sw.x<xMin or sw.x>xMax then
			--sw.SpriteBatch:unbind()
			sw.SpriteBatch:clear()
			table.remove(SeaWeeds,k)
		elseif sw.c~=CurrentWeed then
			--sw.SpriteBatch:unbind()
			sw.SpriteBatch:setImage(Pics.seaweed[CurrentWeed])
			sw.c=CurrentWeed
			--sp:bind()
		end
	end
	local MissingXs={}
	for x=xMin,xMax,BatchWidth do
		missing=true
		for _,sw in ipairs(SeaWeeds) do
			if sw.x==x then
				missing=false
			end
		end
		if missing then
			table.insert(MissingXs,x)
		end
	end
	for _,xMid in ipairs(MissingXs) do
		local sp=love.graphics.newSpriteBatch(Pics.seaweed[CurrentWeed],math.ceil(BatchWidth/4))
		local rng=love.math.newRandomGenerator()
		rng:setSeed(1+math.ceil(xMid/BatchWidth)%12345678)
		for k=1,10 do
			rng:random()
		end
		local xMax=xMid+BatchWidth
		local x=xMid-BatchWidth/2
		local function xIncrease()
			x=math.ceil(x+4+8*rng:random()+(4*rng:random())^4)
		end
		xIncrease()
		while x<xMax do
			xIncrease()
			local scale=0.6+0.4*rng:random()
			sp:add(Pics.seaweed.quad,x,0,0,scale,scale,0,41,0,0)
		end
		sp:setBufferSize(sp:getCount())
		--sp:bind()
		table.insert(SeaWeeds,{x=xMid,SpriteBatch=sp,c=CurrentWeed})
	end
end


local function UpdatePlayer(dt)

	-- Constant
	local rotspeed=tau/64
	local maxspeed=30
	local accel=1
	local decel=3
	local g=2
	local nearfloe=false

	local p=Orca
	p.xlast=p.x-- Save last x pos
	local tr=nil-- Target rotation

	-- Player speed
	if love.mouse.isDown("l") then
		p.speed=math.min(maxspeed,p.speed+accel)
	else
		p.speed=math.max(0,p.speed-decel)
	end

	-- Speed
	if p.y<0 then
		-- Speed in the air
		p.dx=p.dx
		p.dy=p.dy+g
	else
		-- Speed in the water
		p.dx=p.speed*math.cos(p.r)
		p.dy=p.speed*math.sin(p.r)
	end

	-- Player new position
	p.x=p.x+p.dx
	p.y=p.y+p.dy

	-- Adjust which floes to take into account
	AdjustFloesBoundary()

	-- Adjust which seals to take into account
	AdjustSealsBoundary()

	-- Collision with floes
	if p.y<100 and p.y>-100 then
		for f=LeftFloe,RightFloe do
			local fx1,fx2=Floes[f][1],Floes[f][2]
			if p.x+p.size/2>=fx1 and p.x-p.size/2<=fx2 then
				nearfloe=true
				if math.abs(p.y-10)<50 then
					if p.y<0 then
						p.y=math.min(p.y,-40)
					else
						p.y=math.max(p.y,60)
					end
					p.dy=0
					p.speed=math.abs(p.dx)
				end
				if p.y<0 or p.dy<0 then
					tr=(p.r+tau/4)%tau<tau/2 and 0 or tau/2
					if p.y<0 then
						rotspeed=rotspeed/4
					else
						rotspeed=rotspeed*2
					end
				end
				if p.dy<-0.8*maxspeed then
					BreakFloe(f,p.x,40)
					break
				end
			end
		end
	else
		if p.facing==1 and math.abs(p.r%tau-tau/2)<tau/8 then
			p.facing=-1
		end
		if p.facing==-1 and math.abs(p.r%tau)<tau/8 then
			p.facing=1
		end
	end

	-- Collision with sea floor
	if p.y>SeaFloor-60 then
		p.y=SeaFloor-60
		p.dy=0
		p.speed=math.abs(p.dx)
	end

	-- Target rotation
	if not tr then
		-- If not hitting a floe
		if p.y<0 then
			-- In the air, slowly turn downward
			tr=tau/4
			rotspeed=rotspeed/2
		else
			-- If in the water, follow the mouse pointer
			local mx,my=love.mouse.getPosition()
			local wx,wy=love.window.getWidth(),love.window.getHeight()
			tr=math.atan2(my-wy/2,mx-wx/2)
			if nearfloe then
				if p.y>0 and math.abs((tr+tau/2)%tau-tau/4)<tau/4 then
					tr=(p.r+tau/4)%tau<tau/2 and 0 or tau/2
				end
			end
		end
	end

	-- To smoothly reset the orca at a given height to help recording looping animated gif
	if ScreenShotting and love.mouse.isDown("r") then
		local mx,my=love.mouse.getPosition()
		local wx,wy=love.window.getWidth(),love.window.getHeight()
		tr=math.atan2(300-p.y,200*p.facing)
	end

	-- Make the rotation tries to match the target rotation
	if math.abs((tr-p.r+4*tau)%tau-tau/2)>=tau/2-rotspeed then
		p.r=tr
	elseif (tr-p.r+4*tau)%tau>tau/2 then
		p.r=p.r-rotspeed
	else
		p.r=p.r+rotspeed
	end

	-- Update the bubble emitter
	if love.getVersion then
		Bubbles:moveTo(p.x,p.y)
	else
		Bubbles:setPosition(p.x,p.y)
	end
	Bubbles:setEmissionRate(p.speed)
	Bubbles:update(dt)

end


local function UpdateSeals(dt)
	local SealDef={
		Gravity=2,
		MaxRot=tau/32,
		MaxSpeed={Ground=15,Sea=25},
		Acceleration={Ground=1,Sea=1},
		Sight={
			Air={Front=700,Rear=100},
			Sea={Front=2000,Rear=800},}
		}

	local xMouth,yMouth=GetOrcaMouth()

	for n=RightSeal,LeftSeal,-1 do

		-- Current seal in a
		local a=Seals[n]

		-- Find out if the orca is in the seal sight
		local SeeOrca=false
		if a.y<0 then
			-- In the air
			if Orca.y<45 and math.abs(a.x-Orca.x)<SealDef.Sight.Air.Front then
				if math.abs(a.x-Orca.x)<SealDef.Sight.Air.Rear or a.facing==sign(Orca.x-a.x) then
					SeeOrca=true
				end
			end
		else
			-- In water
			if Orca.y<-45 and math.abs(a.x-Orca.x)<SealDef.Sight.Sea.Front then
				if math.abs(a.x-Orca.x)<SealDef.Sight.Sea.Rear or a.facing==sign(Orca.x-a.x) then
					SeeOrca=true
				end
			end
		end

		-- Seal activate if it seas orca
		if not a.active and SeeOrca then
			ActivateSeal(a)
		end

		-- If seal active
		if a.active then

			-- Invert direction if facing and seeing orca
			if SeeOrca and a.facing==sign(Orca.x-a.x) then
				a.facing=-a.facing
				a.r=InvertAngle(a.r)
				a.speed=0
			end

			local tr=nil-- Target rotation
			local rotspeed=SealDef.MaxRot
			local nearfloe=false

			-- New position
			a.x=a.x+a.dx
			a.y=a.y+a.dy

			-- Find closest floe
			local cf=math.ceil(#Floes/2)
			local i=2
			while i<#Floes do
				i=i*2
			end
			repeat
				if Floes[cf][2]<a.x then
					cf=math.min(#Floes,cf+i)
				elseif Floes[cf][1]>a.x then
					cf=math.max(1,cf-i)
				end
				i=math.floor(i/2)
			until i==0
			while Floes[cf][1]>a.x and cf>=2 do
				cf=cf-1
			end
			while Floes[cf][2]<a.x and cf~=#Floes do
				cf=cf+1
			end

			-- Collision with floes
			if a.y<50 and a.y>-50 then
				for f=cf-1,cf+1 do
					local fx1,fx2=Floes[f][1],Floes[f][2]
					if a.x+a.size/2>=fx1 and a.x-a.size/2<=fx2 then
						nearfloe=true
						if math.abs(a.y-5)<25 then
							if a.y<0 then
								a.y=math.min(a.y,-10)
							else
								a.y=math.max(a.y,20)
							end
							a.dy=0
							a.speed=math.abs(a.dx)
						end
						if a.y<0 or a.dy<0 then
							tr=(a.r+tau/4)%tau<tau/2 and 0 or tau/2
							if a.y<0 then
								rotspeed=SealDef.MaxRot/4
							else
								rotspeed=SealDef.MaxRot*2
							end
						end
					end
				end
			else
				if a.facing==1 and math.abs(a.r%tau-tau/2)<tau/8 then
					a.facing=-1
				end
				if a.facing==-1 and math.abs(a.r%tau)<tau/8 then
					a.facing=1
				end
			end

			-- Speed
			if a.y<0 then
				if nearfloe then
					-- Speed on a floe
					a.speed=math.min(SealDef.MaxSpeed.Ground,a.speed+SealDef.Acceleration.Ground)
					a.dx=a.speed*math.cos(a.r)
					a.dy=0
				else
					-- Speed in the air
					a.dx=a.dx
					a.dy=a.dy+SealDef.Gravity
					a.speed=math.sqrt(a.dx^2+a.dy^2)
				end
			else
				-- Speed in the water
				a.speed=math.min(SealDef.MaxSpeed.Sea,a.speed+SealDef.Acceleration.Sea)
				a.dx=a.speed*math.cos(a.r)
				a.dy=a.speed*math.sin(a.r)
			end

			-- Collision with sea floor
			if a.y>SeaFloor-30 then
				a.y=SeaFloor-30
				a.dy=0
				a.speed=math.abs(a.dx)
			end

			-- Target rotation
			if not tr then
				-- If not hitting a floe
				if a.y<0 then
					-- In the air, slowly turn downward
					tr=tau/4
					rotspeed=rotspeed/2
				else
					-- If in the water, avoid orca
					tr=math.atan2(a.x-Orca.x,a.y-Orca.y+(a.y<SeaFloor/2 and 100 or -200))
					if nearfloe then
						if a.y>0 and math.abs((tr+tau/2)%tau-tau/4)<tau/4 then
							tr=(a.r+tau/4)%tau<tau/2 and 0 or tau/2
						end
					end
				end
			end

			-- Make the rotation tries to match the target rotation
			if math.abs((tr-a.r+4*tau)%tau-tau/2)>=tau/2-rotspeed then
				a.r=tr
			elseif (tr-a.r+4*tau)%tau>tau/2 then
				a.r=a.r-rotspeed
			else
				a.r=a.r+rotspeed
			end


		end

		-- Seal get eaten if too close from mouth
		if (a.x-xMouth)^2+(a.y-yMouth)^2<20^2 then
			BloodBlots:setPosition(a.x,a.y)
			BloodBlots:emit(99)
			Score=Score+1
			table.remove(Seals,n)
		end

	end
	BloodBlots:update(dt)
end

function love.update(dt)
	local ForcedUpdateRate=0.030
	UpdatePlayer(ForcedUpdateRate)
	UpdateSeals(ForcedUpdateRate)
	IceShards:update(ForcedUpdateRate)
	UpdateSeaWeeds()
	if love.event.quit then
		if ScreenShotting then
			love.timer.sleep(0.1) -- slidehow speed for screenshoting
		else
			love.timer.sleep(ForcedUpdateRate) -- 0.8.0
		end
	else
		love.timer.sleep(30) -- 0.7.2
	end
end














